// Copyright (c) Tech Soft 3D, Inc.
//
// The information contained herein is confidential and proprietary to Tech Soft 3D, Inc.,
// and considered a trade secret as defined under civil and criminal statutes.
// Tech Soft 3D, Inc. shall pursue its civil and criminal remedies in the event of
// unauthorized use or misappropriation of its trade secrets.  Use of this information
// by anyone other than authorized employees of Tech Soft 3D, Inc. is granted only under
// a written non-disclosure agreement, expressly prescribing the scope and manner of such use.

// HOpConstructRectangle.cpp : implementation of the HOpConstructRectangle class
//

#include <math.h>
#include <string.h>

#include "HTools.h"
#include "HBaseModel.h"
#include "HBaseView.h"
#include "HBaseOperator.h"
#include "HEventInfo.h"
#include "HOpConstructRectangle.h"
#include "HUtility.h"




/////////////////////////////////////////////////////////////////////////////
// HOpConstructRectangle.cpp : implementation of the HOpConstruction class
//
// Operator for rubberbanding a rectangle based on mouse input.
//
// Left button down:
//		Find first position
//
// Left Button down and motion:
//		Rubber band box is inserted in HOOPS "windowspace" segment 
//
//		"Windowspace" is a HOOPS segment with a camera that has a streteched projection
//		and a field width and height of 2 units. Thus window coordinates and world 
//		coordinates with z=0 value are the same. This is convenient for drawing 
//		temporary graphics.
//
//
// Left Button Up:
//		Store points for Rectangle in window space; 
//
//


HOpConstructRectangle::HOpConstructRectangle(HBaseView* view, int DoRepeat, int DoCapture, bool UseCenterMarker) : HBaseOperator(view, DoRepeat, DoCapture)
{
	m_bUseCenterMarker = UseCenterMarker;
}

HOpConstructRectangle::~HOpConstructRectangle()
{
}

HBaseOperator * HOpConstructRectangle::Clone()
{
	return (HBaseOperator *)new HOpConstructRectangle(m_pView);
}

/*!
	GetName assigns the name of the operator--"Construct Rectangle"--to a character pointer.
	\param *name A pointer to a character string
*/
void HOpConstructRectangle::GetName(char *name) { strcpy(name,"Construct Rectangle"); }


/////////////////////////////////////////////////////////////////////////////
// HOpConstructRectangle message handlers


/*!
	OnLButtonDown records the first mouse position and initiates the rectangle-drawing mechanism.
	\param event An HEventInfo object containing information about the current event.
	\return An integer value indicating success or failure.
*/
int HOpConstructRectangle::OnLButtonDown(HEventInfo &event)
{	
	m_ptFirst = event.GetMouseWindowPos();

	if (!m_bOpStarted) 
	{
		m_bOpStarted = true;
		m_bRectangleExists = false;
	}
		
	return (HOP_OK);
}


/*!
	OnLButtonDownAndMove draws a rubberbanded line from the first point in the rectangle to the current mouse position. 
	Note that the basic drawing work is done by HUtility::InsertRectangle, while OnLButtonDownAndMove keeps track of 
	the current points and draws the rubberband centroid.
	\param event An HEventInfo object containing information about the current event.  
	\return An integer value indicating success or failure.
*/
int HOpConstructRectangle::OnLButtonDownAndMove(HEventInfo &event)
{
	if (!m_bOpStarted) return HBaseOperator::OnLButtonDownAndMove(event);
  	
	// Get the new mouse position
    m_ptNew = event.GetMouseWindowPos();

	// Insert a box on viewplane using the first and new mouse positions
    HC_Open_Segment_By_Key(m_pView->GetWindowspaceKey());

		HC_Flush_Geometry (".");
		HUtility::InsertRectangle (".", m_ptFirst.x, m_ptFirst.y, m_ptNew.x, m_ptNew.y);

		if (m_bUseCenterMarker) 
		{
		HPoint center;

			center.x = 0.5F * (m_ptFirst.x + m_ptNew.x);
			center.y = 0.5F * (m_ptFirst.y + m_ptNew.y);
			center.z = 0.5F * (m_ptFirst.z + m_ptNew.z);

			HC_Insert_Marker (center.x, center.y, center.z);
			HC_Set_Visibility ("markers = on");	
		}

	HC_Close_Segment();
    m_pView->Update();

    return (HOP_OK);
}


/*!
	OnLButtonUp finalizes the size of the rectangle and cleans up.
	\param event An HEventInfo object containing information about the current event.
	\return An integer value indicating readiness for the next operation.
*/
int HOpConstructRectangle::OnLButtonUp(HEventInfo &event)
{
	if(!m_bOpStarted) 
		return HBaseOperator::OnLButtonDownAndMove(event);

    m_ptNew = event.GetMouseWindowPos();

	//make certain that new_point != first_point 
	if (m_ptNew.x == m_ptFirst.x &&
		m_ptNew.y == m_ptFirst.y)
		return(HOP_CANCEL);

	// Store Rectangle Points in window space
	m_ptRectangle[0] = m_ptFirst;
	m_ptRectangle[1] = m_ptNew;
	m_bRectangleExists = true;

	// Clean up HOOPS Segment Tree
	HC_Open_Segment_By_Key(m_pView->GetWindowspaceKey());
		HC_Flush_Geometry (".");
		HC_UnSet_One_Visibility ("markers");
	HC_Close_Segment();

    m_pView->Update(); 
    m_bOpStarted = false; 

    return(HOP_READY);
}

